﻿Imports OPSSDK
Imports OPSSDKCommon.Model.Message
Imports Ozeki.VoIP


Module Module1
    Public prompt As String = String.Empty
    Public client As OPSSDK.OpsClient
    Public phoneCall As OPSSDK.ICall
    Public apiExtension As OPSSDK.IAPIExtension

    Sub Main()

        ShowGreetingMessage()
        ReadLoginInfos()
        ReadCommands()
    End Sub

    Sub ShowGreetingMessage()
        Console.WriteLine("This is a simple Ozeki Phone System XE demo written in Visual Basic.")
        Console.WriteLine("It can be used to setup calls, send sms's and receive")
        Console.WriteLine("corresponding delivery reports.")
        Console.WriteLine("The source code of this demo can be downloaded from: ")
        Console.WriteLine("http://www.ozekiphone.com/c-net-voip-developers-740.html")
        Console.WriteLine("-------------------------------------------------------------------------------")
        Console.WriteLine()
    End Sub

    Sub ReadLoginInfos()
        Console.WriteLine("Please enter the IP address of Ozeki Phone System XE.")

        Dim serverAddress = Read("Server address (default: 127.0.0.1)", False)

        If (String.IsNullOrEmpty(serverAddress)) Then
            serverAddress = "127.0.0.1"
        End If

        Console.WriteLine("Please enter the username and password of the user created in Ozeki Phone System XE")
        Dim username = Read("Username (default: admin)", False)

        If (String.IsNullOrEmpty(username)) Then
            username = "admin"
        End If

        Dim password = Read("Password", True)

        Login(serverAddress, username, password)
    End Sub

    Function Read(ByVal inputName As String, ByVal readWhileEmpty As Boolean)
        Console.Write(prompt + "" + inputName + ": ")

        While (True)
            Dim input As String = Console.ReadLine()

            If (Not readWhileEmpty) Then
                Return input
            End If

            If (Not String.IsNullOrEmpty(input)) Then
                Return input
            End If

            Console.WriteLine(inputName + " cannot be empty.")
            Console.Write(inputName + ":")
        End While
    End Function

    Sub Login(ByVal serverAddress, ByVal username, ByVal password)
        client = New OPSSDK.OpsClient()

        AddHandler client.ErrorOccurred, AddressOf ErrorOccurred

        Console.WriteLine("Connecting...")

        Dim result = client.Login(serverAddress, username, password)

        If (result) Then
            Console.WriteLine("Successfully connected to {0} with username: {1}.", serverAddress, username)
            apiExtension = GetAPIExtension()

            prompt = String.Format("[{0}@{1}]> ", apiExtension.ExtensionId, serverAddress)
            AddHandler apiExtension.IncomingCall, AddressOf apiextension_IncomingCall
            AddHandler apiExtension.MessageDelivered, AddressOf apiExtension_MessageDelivered
            AddHandler apiExtension.MessageSubmitted, AddressOf apiExtension_MessageSubmitted
            AddHandler apiExtension.MessageReceived, AddressOf apiExtension_MessageReceived

        Else
            Console.WriteLine("Connection fail try again.")
            Console.WriteLine("Please check whether the IP address is correct and the given user has right to use OPS SDK. This can be checked under PBX features/Preferences/User access profile.")
            ReadLoginInfos()
        End If
    End Sub

    Sub ErrorOccurred(ByVal sender As Object, ByVal e As OPSSDK.ErrorInfo)
        Console.WriteLine(e.Message)
    End Sub

    Sub apiextension_IncomingCall(ByVal sender As Object, ByVal e As Ozeki.VoIP.VoIPEventArgs(Of OPSSDK.ICall))
        Console.WriteLine("Incoming call received from: " + e.Item.OtherParty)

        If (Not IsNothing(phoneCall)) Then
            Console.WriteLine("Another call in progress. Incoming call rejected automatically.")
            e.Item.Reject()
            Return
        End If

        phoneCall = e.Item

        InitCall()
        phoneCall.Accept()
    End Sub

    Sub InitCall()
        AttachCallHandlers()
        SubscribeCallEvents()
    End Sub

    Sub AttachCallHandlers()
        Dim speaker = Ozeki.Media.MediaHandlers.Speaker.GetDefaultDevice()

        If (Not IsNothing(speaker)) Then
            phoneCall.ConnectAudioReceiver(speaker)
            speaker.Start()
        End If

        Dim microphone = Ozeki.Media.MediaHandlers.Microphone.GetDefaultDevice()

        If (Not IsNothing(microphone)) Then
            phoneCall.ConnectAudioSender(microphone)
            microphone.Start()
        End If
    End Sub

    Sub SubscribeCallEvents()
        AddHandler phoneCall.CallStateChanged, AddressOf call_CallStateChanged
        AddHandler phoneCall.CallErrorOccurred, AddressOf call_CallErrorOccured
    End Sub

    Function GetAPIExtension()
        Console.WriteLine("Select API extension.")

        Dim id = Read("API Extension phone number", True)

        Dim extension = client.GetAPIExtension(id)

        If (IsNothing(extension)) Then
            Console.WriteLine("Invalid API Extension phone number.")
            Return GetAPIExtension()
        End If

        Return extension
    End Function

    Private Sub call_CallErrorOccured(ByVal sender As Object, ByVal e As Ozeki.VoIP.VoIPEventArgs(Of OPSSDKCommon.Model.Call.CallError))
        phoneCall = Nothing
        Console.WriteLine("Call error occurred: {0}", e.Item)
    End Sub

    Sub call_CallStateChanged(ByVal sender As Object, ByVal e As Ozeki.VoIP.VoIPEventArgs(Of OPSSDKCommon.Model.Call.CallState))
        If (e.Item.IsCallEnded()) Then
            phoneCall = Nothing
        End If

        Console.WriteLine("Call state changed: {0}", e.Item)
    End Sub

    Sub ReadCommands()
        Console.WriteLine("This demo features the following commands.")
        Console.WriteLine("The first word specifies the name of the command followed by the mandatory command parameters.")

        ShowHelp()

        While (True)
            Console.Write(prompt)
            Dim command = Console.ReadLine()

            If (String.IsNullOrEmpty(command)) Then
                Continue While
            End If

            Dim parameters = GetStringParts(command)

            Dim commandName = parameters(0).ToLower()

            parameters.RemoveAt(0)

            Select Case commandName
                Case "sendsms"
                    SendSMSMessage(parameters)

                Case "call"
                    OutCall(parameters)

                Case "callandsay"
                    CallAndSay(parameters)

                Case "hangup"
                    HangupCall()

                Case "exit"
                    Environment.Exit(0)

                Case "help"
                    ShowHelp()

                Case Else
                    Console.WriteLine("Invalid command.")
            End Select
        End While
    End Sub

    Function GetStringParts(ByVal inputString As String) As List(Of String)
        Dim retVal = New List(Of String)
        Dim currentPart As String = String.Empty
        Dim lexerState As Integer = 0

        For i As Integer = 0 To inputString.Length - 1
            Select Case lexerState
                Case 0
                    If (inputString(i).Equals(" "c)) Then
                        Dim res = currentPart.Trim()

                        If (Not res = String.Empty) Then
                            retVal.Add(currentPart.Trim())
                        End If

                        currentPart = String.Empty

                    Else
                        If (inputString(i).Equals("""")) Then
                            lexerState = 1
                        Else
                            currentPart = currentPart + inputString(i)
                        End If
                    End If
                Case 1
                    If (inputString(i).Equals("""")) Then
                        lexerState = 0
                    Else
                        currentPart = currentPart + inputString(i)
                    End If

            End Select
        Next

        If (Not currentPart = String.Empty) Then
            retVal.Add(currentPart)
        End If

        Return retVal
    End Function

    Sub ShowHelp()
        Console.WriteLine()
        Console.WriteLine("Available commands:")
        Console.WriteLine()

        Console.WriteLine("sendsms <recipient> <content>")
        Console.WriteLine("call <target>")
        Console.WriteLine("callandsay <target> <message>")
        Console.WriteLine("hangup")
        Console.WriteLine("help")
        Console.WriteLine("exit")
        Console.WriteLine()
    End Sub

    Private Sub SendSMSMessage(ByVal parameters As List(Of String))
        If (Not parameters.Count = 2) Then
            Console.WriteLine("Invalid sendsms parameters.")
            Return
        End If

        Dim smsMessage = New OPSSDKCommon.Model.Message.SMSMessage(parameters(0), parameters(1))

        Dim result = apiExtension.SendMessage(smsMessage)

        Dim outMessage As String

        If (IsNothing(result)) Then
            outMessage = String.Empty
        Else
            outMessage = result.RoutingState.ToString()
        End If

        Console.WriteLine("SMS message sent. Message id: {0}, result: {1}", smsMessage.MessageId, outMessage)
    End Sub

    Private Sub apiExtension_MessageDelivered(ByVal sender As Object, ByVal e As OPSSDKCommon.Model.Message.MessageResultEventArgs)
        Console.WriteLine("Message delivery report received. Message id: {0}, result: {1}", e.CallbackId, e.Result)
    End Sub

    Private Sub apiExtension_MessageReceived(ByVal sender As Object, ByVal e As OPSSDKCommon.Model.Message.Message)
        Console.WriteLine("Message received from: {0}, content: {1}", e.Sender, e.Content)
    End Sub

    Private Sub apiExtension_MessageSubmitted(ByVal sender As Object, ByVal e As OPSSDKCommon.Model.Message.MessageResultEventArgs)
        Console.WriteLine("Message submit report received. Message id: {0}, result: {1}", e.CallbackId, e.Result)
    End Sub

    Private Sub OutCall(ByVal parameters As List(Of String))
        If (Not IsNothing(phoneCall)) Then
            Console.WriteLine("Call already in progress. Please hangup the current call.")
            Return
        End If

        If (Not parameters.Count = 1) Then
            Console.WriteLine("Invalid call parameters.")
            Return
        End If

        phoneCall = apiExtension.CreateCall(parameters(0))
        InitCall()
        phoneCall.Start()
    End Sub

    Private Sub CallAndSay(ByVal parameters As List(Of String))
        If (Not IsNothing(phoneCall)) Then
            Console.WriteLine("Call already in progress. Please hangup the current call.")
            Return
        End If

        If (Not parameters.Count = 2) Then
            Console.WriteLine("Invalid callandsay parameters.")
            Return
        End If

        phoneCall = apiExtension.CreateCall(parameters(0))
        SubscribeCallEvents()

        Dim textToSpeech = New Ozeki.Media.MediaHandlers.TextToSpeech()

        AddHandler textToSpeech.Stopped, Sub(sender As Object, e As Ozeki.VoIP.VoIPEventArgs(Of OPSSDKCommon.Model.Call.CallState))
                                             HangupCall()
                                         End Sub
        AddHandler phoneCall.CallStateChanged, Sub(sender As Object, e As Ozeki.VoIP.VoIPEventArgs(Of OPSSDKCommon.Model.Call.CallState))
                                                   If (e.Item.IsInCall()) Then
                                                       phoneCall.ConnectAudioSender(textToSpeech)
                                                       textToSpeech.AddAndStartText(parameters(1))
                                                   End If
                                               End Sub
        phoneCall.Start()
    End Sub

    Private Sub HangupCall()
        If (Not IsNothing(phoneCall)) Then
            phoneCall.HangUp()
            phoneCall = Nothing
        End If
    End Sub
End Module
